﻿//
//              INSERT DATE AND TIME IN SOURCE FILE
//            An AddIn for Pelles C by LD Blake, 2012
//

#define UNICODE  
#define _UNICODE  

#define WIN32_DEFAULT_LIBS
#include <windows.h>
#include <mmsystem.h>
#include <addin.h>

// Locals.
static HANDLE hInst = NULL;
static HWND hMain = NULL;

#define MAX_DATE 80


// insert text into source window
void InsertText(PWCHAR txt)
  { 
    HWND        mdi;  // current window

    mdi = AddIn_GetActiveDocument(hMain);

    // make sure it's text
    if (AddIn_GetDocumentType(mdi) != AID_SOURCE)
      {
        PlaySound(L"SystemAsterisk", NULL, SND_ALIAS | SND_ASYNC);
        return;
      }

    // Insert text
    AddIn_ReplaceSourceSelText(mdi,txt);
  }



// commented timestamp
void InsertTimeStamp(void)
  {
    WCHAR dt[MAX_DATE];  // timestamp string
    
    // add lead comment
    lstrcpy(dt,L"/* ");

    // get short date
    GetDateFormat(LOCALE_USER_DEFAULT,DATE_SHORTDATE,
                    NULL,NULL,&dt[lstrlen(dt)],MAX_DATE - lstrlen(dt));
    // need a space
    lstrcat(dt,L" ");
    // get time
    GetTimeFormat(LOCALE_USER_DEFAULT,TIME_FORCE24HOURFORMAT | TIME_NOTIMEMARKER,
                    NULL,NULL, &dt[lstrlen(dt)],MAX_DATE - lstrlen(dt));

    // add closing comment
    lstrcat(dt,L" */");
    
    // stuff it into the editor
    InsertText(dt);
  }



// short date and time 
void InsertShortDate(void)
  {
    WCHAR dt[MAX_DATE];  // date/time string
    
    // get short date
    GetDateFormat(LOCALE_USER_DEFAULT,DATE_SHORTDATE,NULL,NULL,dt,MAX_DATE);
    // need a space
    lstrcat(dt,L" ");
    // get time
    GetTimeFormat(LOCALE_USER_DEFAULT,TIME_FORCE24HOURFORMAT | TIME_NOTIMEMARKER,
                    NULL,NULL, &dt[lstrlen(dt)],MAX_DATE - lstrlen(dt));

    // stuff it into the editor
    InsertText(dt);
  }



// long date and time 
void InsertLongDate(void)
  {
    WCHAR dt[MAX_DATE];  // date/time string
    
    // get short date
    GetDateFormat(LOCALE_USER_DEFAULT,DATE_LONGDATE,NULL,NULL,dt,MAX_DATE);
    // need a space
    lstrcat(dt,L" ");
    // get time
    GetTimeFormat(LOCALE_USER_DEFAULT,0,
                    NULL,NULL, &dt[lstrlen(dt)],MAX_DATE - lstrlen(dt));

    // stuff it into the editor
    InsertText(dt);
  }



// menu and button commands from POIDE
ADDINAPI void WINAPI AddInCommandEx(int Cmd, LPCVOID Data)
  { 
    switch (Cmd)
      {
        case 100 :
          InsertTimeStamp();
          break;
        case 110 :
          InsertShortDate();
          break;
        case 120 :
          InsertLongDate();
      }
  }



// initialize the addin
BOOL InitAddIn(HWND Win)
  { 
    ADDIN_ADD_COMMAND ac = {0};

    hMain = Win;

    ac.cbSize = sizeof(ac);
    ac.hIcon  = LoadImage(hInst,L"AICON",IMAGE_ICON,16,16,LR_LOADTRANSPARENT); 

    ac.pszText= L"Insert commented timestamp";
    ac.idMenu = AIM_MENU_SOURCE;
    ac.id     = 100;
    AddIn_AddCommand(hMain,&ac); 

    ac.pszText= L"Insert short date and time";
    ac.idMenu = AIM_MENU_SOURCE;
    ac.id     = 110;
    AddIn_AddCommand(hMain,&ac); 

    ac.pszText= L"Insert long date and time";
    ac.idMenu = AIM_MENU_SOURCE;
    ac.id     = 120;
    AddIn_AddCommand(hMain,&ac); 

    return 1;
  }



// main addin procedure
ADDINAPI BOOL WINAPI AddInMain(HWND hwnd, ADDIN_EVENT eEvent)
  { 
    switch (eEvent)
      { 
        case AIE_APP_CREATE:
            return InitAddIn(hwnd); 
         case AIE_APP_DESTROY :
            return AddIn_RemoveCommand(hwnd,100);
        default:
            return TRUE;  
      } 
  }



// dll main entry point
BOOL WINAPI DllMain(HANDLE hDLL, DWORD dwReason, LPVOID lpReserved)
  { 
    hInst = hDLL;
    return TRUE;  
  }


