//                  SDK Keyword Search
//             F1 Search AddIn for Pelles C 
//
//                  By L D Blake, 2012
//

#define UNICODE  
#define _UNICODE  

#define WIN32_DEFAULT_LIBS
#include <windows.h>
#include <shlwapi.h>
#include <addin.h>
#include <wchar.h>

// Locals.
static HANDLE hInst = NULL;
static HWND hMain = NULL;

#define OPENFLT L"Executable Files\0*.exe\0\0"

#ifndef _WIN64
#define INIT_DIR L"C:\\Program Files\\Common Files\\Microsoft Shared"
#define DEXPLORE L"C:\\Program Files\\Common Files\\Microsoft Shared\\Help 9\\dexplore.exe"
#else
#define INIT_DIR L"C:\\Program Files (x86)\\Common Files\\Microsoft Shared"
#define DEXPLORE L"C:\\Program Files (x86)\\Common Files\\Microsoft Shared\\Help 9\\dexplore.exe"
#endif

#define DEF_KEYWORD L"/helpcol MS-Help://MS.W7SDK.1033 /launchfkeywordtopic "
#define DEF_MAIN L"/helpcol MS-Help://MS.W7SDK.1033"

#define REG_HIVE HKEY_CURRENT_USER
#define REG_PATH L"Software\\Pelle Orinius\\PellesC\\SDKSearch"


// get filename to open
BOOL GetFileName(HWND Dlg)
  { 
    OPENFILENAME ofn = {0};    // filename struct
    WCHAR fn[MAX_PATH] = {0};  // returned name


    // precondition struct
    ofn.lStructSize     = sizeof(ofn);
    ofn.lpstrInitialDir = INIT_DIR;
    ofn.hInstance       = hInst;
    ofn.hwndOwner       = Dlg; 
    ofn.lpstrFilter     = OPENFLT;
    ofn.nFilterIndex    = 1;
    ofn.lpstrTitle      = L"Select DEXPLORE.EXE Please...";
    ofn.lpstrFile       = fn;
    ofn.nMaxFile        = MAX_PATH;
    ofn.Flags           = OFN_FILEMUSTEXIST |
                          OFN_PATHMUSTEXIST;
    if (GetOpenFileName(&ofn))
      SetDlgItemText(Dlg,4001,fn); 
    return 0;
  }


// save registry settings
BOOL SaveSettings(HWND Dlg)
  { 
    WCHAR tmp[MAX_PATH +1] = {0};

    // save executable path
    GetDlgItemText(Dlg,4001,tmp,MAX_PATH);

    if (SHSetValue(REG_HIVE,REG_PATH,L"Dexplore",REG_SZ, tmp, (wcslen(tmp) + 1) * sizeof(WCHAR))) 
      { 
        MessageBox(Dlg,L"Error saving Settings",L"OOPS!",MB_ICONWARNING | MB_OK);
        return TRUE; 
      }
  
    // save flags for keyword search 
    GetDlgItemText(Dlg,4003,tmp,MAX_PATH);

    if (SHSetValue(REG_HIVE,REG_PATH,L"Keywords",REG_SZ, tmp, (wcslen(tmp) + 1) * sizeof(WCHAR))) 
      { 
        MessageBox(Dlg,L"Error saving Settings",L"OOPS!",MB_ICONWARNING | MB_OK);
        return TRUE; 
      }


    // save flags for main window
    GetDlgItemText(Dlg,4005,tmp,MAX_PATH);

   if (SHSetValue(REG_HIVE,REG_PATH,L"Main",REG_SZ, tmp, (wcslen(tmp) + 1) * sizeof(WCHAR))) 
      { 
        MessageBox(Dlg,L"Error saving Settings",L"OOPS!",MB_ICONWARNING | MB_OK);
        return TRUE; 
      }

   EndDialog(Dlg, 1);
   return 1; 
   
 }


// get registry settings
BOOL ImportSettings(HWND Dlg)
  { 
    DWORD dt;                   // dummy for data type
    DWORD ds;                   // string size
    WCHAR tmp[MAX_PATH + 1] = {0};

    // get dexplore.exe path
    ds = MAX_PATH * sizeof(WCHAR);
    if (!SHGetValue(REG_HIVE,REG_PATH,L"Dexplore",&dt,tmp,&ds))
      SetDlgItemText(Dlg,4001,tmp);
    else
      SetDlgItemText(Dlg,4001,DEXPLORE);

    // get keyword flags
    ds = MAX_PATH * sizeof(WCHAR);
    if (!SHGetValue(REG_HIVE,REG_PATH,L"Keywords",&dt,tmp,&ds))
      SetDlgItemText(Dlg,4003,tmp);
    else
      SetDlgItemText(Dlg,4003,DEF_KEYWORD);

    // get main window flags
    ds = MAX_PATH * sizeof(WCHAR);
    if (!SHGetValue(REG_HIVE,REG_PATH,L"Main",&dt,tmp,&ds))
      SetDlgItemText(Dlg,4005,tmp);
    else
      SetDlgItemText(Dlg,4005,DEF_MAIN);
 
    return 0; 
  }


// settings dialog message tosser
BOOL CALLBACK SetupTosser(HWND Dlg, UINT Msg, WPARAM wParm, LPARAM lParm)
  { 
    switch (Msg)
      { 
        case WM_INITDIALOG :
          return ImportSettings(Dlg);
        case WM_COMMAND :
          switch (LOWORD(wParm))
            { 
              case IDOK :                   // OK
                return SaveSettings(Dlg);
              case IDCANCEL :               // Cancel
                return EndDialog(Dlg,0);
              case 4002 :
                return GetFileName(Dlg);
              case 4004 :
                return SetDlgItemText(Dlg,4003,DEF_KEYWORD);      
              case 4006 :
                return SetDlgItemText(Dlg,4005,DEF_MAIN);      
              default :
                return 1; 
            }
        default :
          return 0; 
      } 
  }


// Setup dialog launcher
ADDINAPI VOID WINAPI AddInSetup(HWND hWin,LPCVOID Nothing)
  { 
    DialogBox(hInst,L"SETUP",hMain,(DLGPROC) SetupTosser); 
  }



// F1 searches
ADDINAPI BOOL WINAPI AddInHelp(HWND hWin,ADDIN_HELPEVENT Event,LPCVOID Data)
  { 
    WCHAR cmd[MAX_PATH] = {0};        // command string
    WCHAR parm[MAX_PATH + 1] = {0};   // command flags
    DWORD dt;
    DWORD ds;

    // let local search complete first
    if (Event == AIHE_SRC_KEYWORD_FIRST)    
      return FALSE;

    // Assemble command line
    ds = MAX_PATH * sizeof(WCHAR);
    if (SHGetValue(REG_HIVE,REG_PATH,L"Dexplore",&dt,cmd,&ds))
      wcscpy(cmd,DEXPLORE);

     ds = MAX_PATH * sizeof(WCHAR);
    if (SHGetValue(REG_HIVE,REG_PATH,L"Keyword",&dt,parm,&ds))
      wcscpy(parm,DEF_KEYWORD);
    wcscat(parm, L" ");

    // add keyword
    wcscat(parm, (PWCHAR) Data);

        
    ShellExecute(NULL,L"OPEN",cmd,parm,NULL,SW_SHOWNORMAL);
    return TRUE; 
  }  



// menu and button commands from POIDE
ADDINAPI void WINAPI AddInCommandEx(int idCmd, LPCVOID pcvData)
  { 
    WCHAR cmd[MAX_PATH + 1] = {0};  // command string
    WCHAR parm[MAX_PATH + 1] = {0}; // command flags
    DWORD dt;
    DWORD ds;


    // only care about one command value
    if (idCmd != 100)
      return;

    // Assemble command line
    ds = MAX_PATH * sizeof(WCHAR);
    if (SHGetValue(REG_HIVE,REG_PATH,L"Dexplore",&dt,cmd,&ds))
      wcscpy(cmd,DEXPLORE);

     ds = MAX_PATH * sizeof(WCHAR);
    if (SHGetValue(REG_HIVE,REG_PATH,L"Main",&dt,parm,&ds))
      wcscpy(parm,DEF_MAIN);
        
    ShellExecute(NULL,L"OPEN",cmd,parm,NULL,SW_SHOWNORMAL);
  }



// initialize the addin
BOOL InitAddIn(HWND Win)
  { 
    ADDIN_ADD_COMMAND ac = {0};
    hMain = Win;
    ac.cbSize = sizeof(ac);
    ac.hIcon  = LoadImage(hInst,L"SDKF1ICON",IMAGE_ICON,16,16,LR_LOADTRANSPARENT); 
    ac.pszText= L"Open Windows 7 SDK";
    ac.idMenu = AIM_MENU_NOTHING;
    ac.id     = 100;
    return AddIn_AddCommand(hMain,&ac); 
  }



// main addin procedure
ADDINAPI BOOL WINAPI AddInMain(HWND hwnd, ADDIN_EVENT eEvent)
  { 
    switch (eEvent)
      { 
        case AIE_APP_CREATE:
            return InitAddIn(hwnd); 
         case AIE_APP_DESTROY :
            return AddIn_RemoveCommand(hwnd,100);
        default:
            return TRUE;  
      } 
  }



// dll main entry point
BOOL WINAPI DllMain(HANDLE hDLL, DWORD dwReason, LPVOID lpReserved)
  { 
    hInst = hDLL;
    return TRUE;  
  }









