/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                 Workspace Manager AddIn for Pelles POIDE                //
//                     Create and edit workspace files                     //
//                                                                         //
//                            By LD Blake, 2012                            //
/////////////////////////////////////////////////////////////////////////////

// helper functions

#include "Global.h"
#include <shobjidl.h>


// browse for a filename
BOOL GetFileName(HWND Owner, PWCHAR Filter, PWCHAR Result)
  { 
    OPENFILENAME ofn = {0};

    ofn.lStructSize = sizeof(ofn);
    ofn.hInstance   = hMod;
    ofn.hwndOwner   = Owner; 
    ofn.lpstrFilter = Filter;
    ofn.nFilterIndex= 1;
    ofn.lpstrTitle  = L"Select a file";
    ofn.lpstrFile   = Result;
    ofn.nMaxFile    = MAX_PATH;
    ofn.Flags       = OFN_HIDEREADONLY | 
                      OFN_FILEMUSTEXIST |
                      OFN_PATHMUSTEXIST;
    // launch dialog
    if (GetOpenFileName(&ofn))
      return 1;
    return 0; 
  }



// browse for a folder name
BOOL GetFolderName(HWND Owner, PWCHAR Root, PWCHAR Result)
  { 
    BROWSEINFO    bi = {0};   // info struct
    LPITEMIDLIST  fidl;       // user folder id list
    LPITEMIDLIST  ridl;       // root folder idlist
    LPSHELLFOLDER desktop;    // desktop folder object
    BOOL          ret;

    // IShellFolder 
    if (! SHGetDesktopFolder(&desktop))
      { 
        desktop->lpVtbl->ParseDisplayName(desktop,NULL,NULL,Root,NULL,&ridl,NULL);
        desktop->lpVtbl->Release(desktop); 
      }

    //setup struct
    bi.hwndOwner  = Owner;
    bi.pidlRoot   = ridl;
    bi.lpszTitle  = L"Select or create a folder";
    bi.ulFlags    = BIF_RETURNONLYFSDIRS | BIF_USENEWUI; 

     // show dialog
    fidl = SHBrowseForFolder(&bi);
    if (! fidl)
      return FALSE;

    // get path in Result
    ret =  SHGetPathFromIDList(fidl,Result);
    CoTaskMemFree(fidl); 
    CoTaskMemFree(ridl);
    return ret; 
  }



// get a string from the registry
BOOL RegLoadString(PWCHAR Reg, PWCHAR Result)
  { 
    DWORD sl = MAX_PATH * sizeof(WCHAR);
    // load the string
    if(SHGetValue(REG_HIVE,REG_SETUP,Reg,NULL,Result,&sl))
      return 0;
    return 1; 
  }



// get a dword from the registry
DWORD RegLoadDword(PWCHAR Reg)
  { 
    DWORD cv = 0;  // value
    DWORD sl = sizeof(DWORD);
    // load value
    SHGetValue(REG_HIVE,REG_SETUP,Reg,NULL,&cv,&sl);
    return cv; 
  }


// in place conversion of To path  
BOOL MakeRelativePath(PWCHAR To, PWCHAR From)
  { 
    WCHAR tpath[MAX_PATH] = {0};  // temp holder
 
    if (PathRelativePathTo(tpath,From,FILE_ATTRIBUTE_DIRECTORY,To,FILE_ATTRIBUTE_DIRECTORY))
      { 
        lstrcpy(To,tpath);  
        return 1;
      }
    return 0; // fail
  } 
 


// write WCHAR string to utf8 file
BOOL DiskWrite(PWCHAR Buffer, PWCHAR Name)
  { 
    INT     bsz = 0;      // required text buffer size
    INT     fsz = 0;      // size on disk file
    HANDLE  fh;           // disk file handle
    PCHAR   fb = NULL;    // file buffer 
    DWORD   bw;           // bytes written

    // verify overwrite
    if (PathFileExists(Name))
      { 
        WCHAR msg[300];

        swprintf(msg,300,L"The file \n%ls\nalready exists. Do you want to overwrite it?",Name);
        if (MessageBox(NULL,msg,L"OOPS!",MB_ICONWARNING | MB_YESNO) == IDNO)
          return 0; 
      }

    // get required buffer size
    bsz = WideCharToMultiByte(CP_UTF8,0,Buffer,-1,NULL,0,NULL,NULL);

    // create text buffer
    fb = calloc(bsz,sizeof(BYTE));

    // convert                              NOTE: return value includes terminating null!
    fsz = WideCharToMultiByte(CP_UTF8,0,Buffer,-1,fb,bsz,NULL,NULL) - 1;

    if (fsz < 0)
      { 
        MessageBox(NULL,L"Error converting the file",L"OOPS!",MB_ICONWARNING | MB_OK);
        return 0; 
      }

    // write to disk
    fh = CreateFile(Name,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
    if (fh == INVALID_HANDLE_VALUE)
      { 
        MessageBox(NULL,L"Unable to write to the file",L"OOPS!",MB_OK | MB_ICONERROR);
        return 0; 
      }
    WriteFile(fh,"\xEF\xBB\xBF\r\n",5,&bw,NULL); //bom
    WriteFile(fh,fb,fsz,&bw,NULL);
    FlushFileBuffers(fh);
    CloseHandle(fh);
    return 1; 
  }


