/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                 Workspace Manager AddIn for Pelles POIDE                //
//                     Create and edit workspace files                     //
//                                                                         //
//                            By LD Blake, 2012                            //
/////////////////////////////////////////////////////////////////////////////

// dll and addin functions

#include "Global.h"
#include "Workspace.h"
#include "Project.h"
#include "Setup.h"


// Locals.
HANDLE  hMod = NULL;
HWND    hMain = NULL;
HWND    hProj = NULL;

//////////////////////////////////////////////////////////////////////////////
// Bells and Bobbles, new weapons for your washing machine.
//

BOOL AddWithMain(HWND Win)
  { 
    ADDIN_ADD_COMMAND ac = {0};

    ac.cbSize = sizeof(ac);

    // add to menus
    ac.hIcon  = LoadImage(hMod,L"WKSMAN",IMAGE_ICON,16,16,LR_LOADTRANSPARENT); 
    ac.pszText= L"Create or Manage Workspaces";
    ac.idMenu = AIM_MENU_FILE;
    ac.id     = 100;
    AddIn_AddCommand(Win,&ac);     
    ac.hIcon  = LoadImage(hMod,L"NEWPRJ",IMAGE_ICON,16,16,LR_LOADTRANSPARENT); 
    ac.pszText= L"Create a New Project";
    ac.idMenu = AIM_MENU_FILE;
    ac.id     = 110;
    AddIn_AddCommand(Win,&ac);     
  
    if (RegLoadDword(L"WorkspaceManager"))
      { 
        ac.hIcon  = LoadImage(hMod,L"WKSMAN",IMAGE_ICON,16,16,LR_LOADTRANSPARENT); 
        ac.pszText= L"Create or Manage Workspaces";
        ac.idMenu = AIM_MENU_NOTHING;
        ac.id     = 120;
        AddIn_AddCommand(Win,&ac); 
      }    

    if (RegLoadDword(L"NewProject"))
      { 
        ac.hIcon  = LoadImage(hMod,L"NEWPRJ",IMAGE_ICON,16,16,LR_LOADTRANSPARENT); 
        ac.pszText= L"Create a new project";
        ac.idMenu = AIM_MENU_NOTHING;
        ac.id     = 130;
        AddIn_AddCommand(Win,&ac); 
      }
    return 1; 
  }



BOOL RemoveWithMain(HWND Win)
  { 
    AddIn_RemoveCommand(Win,100);
    AddIn_RemoveCommand(Win,110);
    AddIn_RemoveCommand(Win,120);
    AddIn_RemoveCommand(Win,130);
    return 1; 
  }



BOOL AddWithProject(HWND Win)
  { 
    ADDIN_ADD_COMMAND ac = {0};

    ac.cbSize = sizeof(ac);

    if (RegLoadDword(L"AddProjects"))
      { 
        ac.hIcon  = LoadImage(hMod,L"ADDPRJ",IMAGE_ICON,16,16,LR_LOADTRANSPARENT); 
        ac.pszText= L"Add an existing project";
        ac.idMenu = AIM_MENU_NOTHING;
        ac.id     = 200;
        AddIn_AddCommand(Win,&ac); 
      }    

    if (RegLoadDword(L"Dependencies"))
      { 
        ac.hIcon  = LoadImage(hMod,L"PRJDEP",IMAGE_ICON,16,16,LR_LOADTRANSPARENT); 
        ac.pszText= L"Set workspace dependencies";
        ac.idMenu = AIM_MENU_NOTHING;
        ac.id     = 210;
        AddIn_AddCommand(Win,&ac); 
      } 
   
    if (RegLoadDword(L"ProjectOptions"))
      { 
        ac.hIcon  = LoadImage(hMod,L"PRJOPTS",IMAGE_ICON,16,16,LR_LOADTRANSPARENT); 
        ac.pszText= L"Set project options";
        ac.idMenu = AIM_MENU_NOTHING;
        ac.id     = 220;
        AddIn_AddCommand(Win,&ac); 
      }
      
    if (RegLoadDword(L"WorkFolder"))
      { 
        ac.hIcon  = LoadImage(hMod,L"WORKFLDR",IMAGE_ICON,16,16,LR_LOADTRANSPARENT); 
        ac.pszText= L"Open project work folder";
        ac.idMenu = AIM_MENU_NOTHING;
        ac.id     = 230;
        AddIn_AddCommand(Win,&ac); 
      }
    
    if (RegLoadDword(L"ResultFolder"))
      { 
        ac.hIcon  = LoadImage(hMod,L"RESFLDR",IMAGE_ICON,16,16,LR_LOADTRANSPARENT); 
        ac.pszText= L"Open project result folder";
        ac.idMenu = AIM_MENU_NOTHING;
        ac.id     = 240;
        AddIn_AddCommand(Win,&ac); 
      }    

    if (RegLoadDword(L"ResultShell"))
      {   
        ac.hIcon  = LoadImage(hMod,L"RESCMD",IMAGE_ICON,16,16,LR_LOADTRANSPARENT); 
        ac.pszText= L"CMD shell";
        ac.idMenu = AIM_MENU_NOTHING;
        ac.id     = 250;
        AddIn_AddCommand(Win,&ac); 
      }    
    return 1; 
  }



BOOL RemoveWithProject(HWND Win)
  {   
    AddIn_RemoveCommand(Win,200);
    AddIn_RemoveCommand(Win,210);
    AddIn_RemoveCommand(Win,220);
    AddIn_RemoveCommand(Win,230);
    AddIn_RemoveCommand(Win,240);
    AddIn_RemoveCommand(Win,250);
    return 1; 
  }


/////////////////////////////////////////////////////////////////////////////
// AddIn commands
//
VOID AddNewProject(HWND Win)
  { 
    NEWPROJECT nprj = {0};

    // prep struct
    nprj.Parent = Win;
    RegLoadString(L"RootFolder",nprj.RootFolder);

    // get new project
    if (! NewProject(&nprj))
      return;

    // add to current workspace?
    if(hProj)
     if(MessageBox(Win,L"Add project to current workspace?",L"Info",
                    MB_ICONQUESTION | MB_YESNO) == IDNO)
        AddIn_SendIDECommand(hMain, AIC_FILE_CLOSEPROJ);  

    // open project file
    if(! hProj)
      AddIn_OpenDocument(hMain,AID_PROJECT,nprj.Path); 
    else
      AddIn_AddWorkspaceProject(hProj,nprj.Path);

    AddIn_SetActiveProject(hProj, nprj.Name);

    if(nprj.Index == 9)  // resource dll
      AddIn_SendIDECommand(hMain, AIC_FILE_NEWRESOURCE);
    else
      AddIn_SendIDECommand(hMain, AIC_FILE_NEWSOURCE);
  }

        

// open project work folder
VOID OpenWorkFolder(void)
  { 
    WCHAR rf[MAX_PATH] = {0};

    AddIn_GetProjectSymbol(hProj,L"POC_PROJECT_WORKPATH",rf,MAX_PATH);
    ShellExecute(NULL,L"OPEN",L"Explorer.exe",rf,NULL,SW_SHOWNORMAL); 
  }

    
    
// open project result folder
VOID OpenResultFolder(void)
  { 
    WCHAR rf[MAX_PATH] = {0};

    AddIn_GetProjectSymbol(hProj,L"POC_PROJECT_RESULTDIR",rf,MAX_PATH);
    ShellExecute(NULL,L"OPEN",L"Explorer.exe",rf,NULL,SW_SHOWNORMAL); 
  }


    
// open command shell
VOID OpenCMDShell(void)
  { 
    WCHAR rf[MAX_PATH] = {0};

    AddIn_GetProjectSymbol(hProj,L"POC_PROJECT_RESULTDIR",rf,MAX_PATH);
    ShellExecute(NULL,L"OPEN",L"CMD.EXE",NULL,rf,SW_SHOWNORMAL); 
  }



//////////////////////////////////////////////////////////////////////////////
// AddIn Functions
//

// setup functions
ADDINAPI void WINAPI AddInSetup(HWND hwnd,LPCVOID pcvData)
  { 
    RemoveWithMain(hMain);
    RemoveWithProject(hProj);

    // run setup  
    ShowSetup(hwnd); 

    // mess up the toolbars
    AddWithMain(hMain);

    if(hProj)
      AddWithProject(hProj); 
  }



// menu and button commands from POIDE
ADDINAPI void WINAPI AddInCommandEx(int Cmd,LPCVOID Data)
  { 
    switch(Cmd)
      { 
        case 100 :
        case 120 :
          EditWorkspace(hMain);
          return;
        case 110 :
        case 130 :
          AddNewProject(hMain);
          return;
        case 200 :
          AddIn_SendIDECommand(hMain,AIC_PROJ_ADDPROJECT);
          return; 
        case 210 :
          AddIn_SendIDECommand(hMain,AIC_PROJ_DEPENDENCIES);
          return; 
        case 220 :
          AddIn_SendIDECommand(hMain,AIC_PROJ_SETTINGS);
          return; 
        case 230 :
          OpenWorkFolder();
          return;
        case 240 :
          OpenResultFolder();
          return;
        case 250 :
          OpenCMDShell();
          return;
         default :
          return ; 
      } 
  }



BOOL InitAddIn(HWND Win)
  { 
    WCHAR incs[MAX_PATH] = {0};

    hMain = Win;
//    CoInitialize(NULL);
    AddWithMain(Win);

    // get bin folder
    AddIn_GetIDEFolder(hMain, ADDIN_FOLDER_BIN, incs, MAX_PATH);
    
    // back up one level (should be main pellesc folder)
    PathRemoveFileSpec(incs);

    // look for <pellesC>\inc
    PathAddBackslash(incs);
    wcscat(incs, L"inc");
    if(! PathIsDirectory(incs))
      CreateDirectory(incs, NULL);

    // now check <pellesC>\inc\ce 
    PathAddBackslash(incs);
    wcscat(incs, L"ce");
    if(! PathIsDirectory(incs))
      CreateDirectory(incs, NULL);
    return 1; 
  }



BOOL KillAddIn(HWND Win)
  { 
    hMain = NULL;  
    RemoveWithMain(Win); 
    return 1; 
  }



BOOL InitProject(HWND Win)
  { 
    if (hProj)
      return 1;
    hProj = Win;
    AddWithProject(Win);
    return 1; 
  }



BOOL KillProject(HWND Win)
  { 
    hProj = NULL;
    RemoveWithProject(Win);
    return 1; 
  }



// main addin procedure
ADDINAPI BOOL WINAPI AddInMain(HWND hwnd, ADDIN_EVENT eEvent)
  { 
    switch (eEvent)
      { 
        case AIE_APP_CREATE:
            return InitAddIn(hwnd); 
        case AIE_APP_DESTROY :
            return KillAddIn(hwnd);
        case AIE_PRJ_CREATE :
            return InitProject(hwnd);
        case AIE_PRJ_DESTROY :
            return KillProject(hwnd);
        default:
            return 1;  
      } 
  }


//////////////////////////////////////////////////////////////////////////////
// dll main attach point
//

BOOL WINAPI DllMain(HANDLE hDLL, DWORD dwReason, LPVOID lpReserved)
  { 
    switch (dwReason)
      { 
        case DLL_PROCESS_ATTACH:
          hMod = hDLL;
          return TRUE;
        case DLL_PROCESS_DETACH:
          return TRUE;
        default:
          return TRUE;  
      } 
  }

