/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                 Workspace Manager AddIn for Pelles POIDE                //
//                     Create and edit workspace files                     //
//                                                                         //
//                            By LD Blake, 2012                            //
/////////////////////////////////////////////////////////////////////////////

// super nifty new New Project dialog

#include "Global.h"
#include "Project.h"

// number of project types
#ifdef _WIN64
#define TYPES_COUNT 19
#else
#define TYPES_COUNT 15
#endif //_WIN64
 


pNEWPROJECT NewPrj;  // address of new project struct

///////////////////////////////////////////////////////////////////////////////
// initializer for project types list
//

// one type
typedef struct t_PROJECTS
  { INT   Type;         // value
    TCHAR Name[32]; }   // name
    PROJECTS, *pPROJECTS;



// project names and values
const PROJECTS PrjList[TYPES_COUNT] =                                         
  { 
    0,  L" ",                           
#ifdef _WIN64                           // res   template     type
    1,  L"64bit Windows GUI Program",   // 8001  w64gui.txt   10 
    2,  L"64bit Console Program",       // 8002  w64con.txt   13
    3,  L"64bit Dynamic Library",       // 8003  w64dll.txt   11
    4,  L"64bit Static Library",        // 8004  w64lib.txt   12
#endif //_WIN64  
    5,  L"32bit Windows GUI Program",   // 8005  w32gui.txt   0
    6,  L"32bit Console Program",       // 8006  w32con.txt   3
    7,  L"32bit Dynamic Library",       // 8007  w32dll.txt   1
    8,  L"32bit Static Library",        // 8008  w32lib.txt   2
    9,  L"Windows Resource DLL",        // 8009  resdll.txt   1
    10, L"Pocket PC GUI Program",       // 8010  ppcgui.txt   4
    11, L"Pocket PC Dynamic Library",   // 8011  ppcdll.txt   5
    12, L"Pocket PC Static Library",    // 8012  ppclib.txt   6
    13, L"Smart Phone GUI Program",     // 8013  sphgui.txt   7
    14, L"Smart Phone Dynamic Library", // 8014  sphdll.txt   8
    15, L"Smart Phone Static Library",  // 8015  sphlib.txt   9
    16, L"Windows Installer",           // 8016  winins.txt   50
    17, L"Pocket PC Installer",         // 8017  ppcins.txt   51
    18, L"Smart Phone Installer"        // 8018  sphins.txt   52
  };


///////////////////////////////////////////////////////////////////////////////
// PPJ file creation
//

// format string for top of project file
const PWCHAR PrjFmt =
  {
    L"# Project file generated by Workspace Manager AddIn\r\n\r\n" 
    L"POC_PROJECT_VERSION = 6.00#\r\n" 
    L"POC_PROJECT_OUTPUTDIR = %ls_out#\r\n"       // <name>_out
    L"POC_PROJECT_RESULTDIR = %ls\\#\r\n"         // result directory  
    L"POC_PROJECT_ARGUMENTS = #\r\n" 
    L"POC_PROJECT_EXECUTOR = #\r\n" 
    L"POC_PROJECT_WORKPATH = .\\#\r\n"
  };



// write project file to disk
BOOL SaveProjectFile(HWND Dlg)
  { 
    WCHAR   name[MAX_NAME];     // project name
    WCHAR   path[MAX_PATH];     // project path
    WCHAR   res[MAX_PATH];      // result path
    PWCHAR  fbuf;               // file buffer
    INT     index;              // project selected

    // project name
    GetDlgItemText(Dlg,4000,name,MAX_NAME);
    if (lstrlen(name) < 1)
      { 
        MessageBox(Dlg,L"Please name your project",L"OOPS!",MB_OK | MB_ICONWARNING);
        return 0; 
      }

    // project path
    GetDlgItemText(Dlg,4003,path,MAX_PATH);
     if (lstrlen(path) < 3)
      { 
        MessageBox(Dlg,L"Please select a project folder",L"OOPS!",MB_OK | MB_ICONWARNING);
        return 0; 
      }
    SetCurrentDirectory(path);  

    // result dir = project dir if empty
    GetDlgItemText(Dlg,4005,res,MAX_PATH);    
    if (lstrlen(res) < 3)
      lstrcpy(res,path); 
    MakeRelativePath(res,path);
 
    // project index
    {
      INT sel = SendDlgItemMessage(Dlg,4001,CB_GETCURSEL,0,0);
      index   = SendDlgItemMessage(Dlg,4001,CB_GETITEMDATA,sel,0); 
    }
    if (index < 1)
      { 
        MessageBox(Dlg,L"Please select a project type.",L"OOPS!",MB_OK | MB_ICONWARNING);
        return 0; 
      }

    // create file buffer
    fbuf = calloc(MAX_PPJ,sizeof(WCHAR));
    if (!fbuf)
      {
        MessageBox(Dlg,L"Project creation failed!",L"OOPS!",MB_OK | MB_ICONERROR);
        return 0; 
      }

    // fill project top form
    swprintf(fbuf,MAX_PPJ,PrjFmt,name,res);

    // Add resource template
    { 
      HRSRC   res1 = FindResource(hMod,MAKEINTRESOURCE(8000 + index),RT_RCDATA);
      HGLOBAL res2 = LoadResource(hMod,res1);
      PWCHAR  tpl  = LockResource(res2);
      INT     sres = SizeofResource(hMod,res1) / sizeof(WCHAR); // convert return value in bytes
      wcsncpy(&fbuf[lstrlen(fbuf)],&tpl[1],sres - 1);           // note res[1] because of BOM
    }

    // build full path
    PathAddBackslash(path);
    wcscat(path, name);
    wcscpy(PathFindExtension(path), L".ppj");
    wcscpy(NewPrj->Path, path);

    // save name and index
    wcscpy(NewPrj->Name, name);
    NewPrj->Index = index;

    // write the file
    {
      BOOL res = DiskWrite(fbuf,path);
      free(fbuf);
      EndDialog(Dlg,res);
      return res;
    }  
  }
  

///////////////////////////////////////////////////////////////////////////////
// Dialog functions
//

// change project path
BOOL GetProjectPath(HWND Dlg,UINT Ctrl, UINT Res)
  { 
    WCHAR path[MAX_PATH];
    PWCHAR root = NULL;

    // is project confined
    if (RegLoadDword(L"ConfineProjects"))
      root = NewPrj->RootFolder;

    // get the folder name
    if (! GetFolderName(Dlg,root,path))
      return 1;

    SetDlgItemText(Dlg,Ctrl,path);

    if (GetWindowTextLength(GetDlgItem(Dlg,Res)) < 1)
      SetDlgItemText(Dlg,Res,path);
    return 1; }  



// change result path 
BOOL GetResultPath(HWND Dlg,UINT Ctrl)
  { 
    WCHAR path[MAX_PATH];
    PWCHAR root = NULL;

    // is project confined
    if (RegLoadDword(L"ConfineProjects"))
      root = NewPrj->RootFolder;

    // get the folder name
    if (! GetFolderName(Dlg,root,path))
      return 1;

    SetDlgItemText(Dlg,Ctrl,path);
    return 1; }  



// load the list box 
BOOL LoadList(HWND Dlg, INT List)
  { 
    INT   item;                  // list item index
    INT   x;                     // counter

    // loop through values inserting them in the list
    for (x = 0; x < TYPES_COUNT; x++)
      { 
        item = SendDlgItemMessage(Dlg,List,CB_ADDSTRING,0,(LPARAM) PrjList[x].Name);
        SendDlgItemMessage(Dlg,List,CB_SETITEMDATA,item,PrjList[x].Type); 
      }
    // set to first item
    SendDlgItemMessage(Dlg,List,CB_SETCURSEL,0,0);
  return 1; }



// dialog startup
BOOL InitDialog(HWND Dlg,LPARAM lParm)
  { 
    NewPrj= (pNEWPROJECT) lParm;

    SendDlgItemMessage(Dlg,4001,CB_SETEXTENDEDUI,1,0);
    LoadList(Dlg,4001);
    SetDlgItemText(Dlg,4003,NewPrj->RootFolder);
    return 1; 
  }



// dialog tosser
BOOL CALLBACK PrjTosser(HWND Dlg, UINT Msg, WPARAM wParm, LPARAM lParm)
  { 
    switch (Msg)
      { 
        case WM_INITDIALOG :        // initialization
          return InitDialog(Dlg,lParm);
        case WM_COMMAND :           // buttons
          switch (LOWORD(wParm))
            { 
              case 4002 :           // project directory
                return GetProjectPath(Dlg,4003,4005);
              case 4004 :           // result directory
                return GetResultPath(Dlg,4005);
              case IDOK :           // Save
                return SaveProjectFile(Dlg);
              case IDCANCEL :       // Cancel
                return EndDialog(Dlg, 0);  
              default :
                return 0; 
            }
        default :
          return 0; 
      } 
  } 



// launch the new project dialog
BOOL NewProject(pNEWPROJECT NewPPJ)
  { 
    return DialogBoxParam(hMod,L"NEWPRJ",NewPPJ->Parent,(DLGPROC) PrjTosser,(LPARAM)NewPPJ); 
  } 

