/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                 Workspace Manager AddIn for Pelles POIDE                //
//                     Create and edit workspace files                     //
//                                                                         //
//                            By LD Blake, 2012                            //
/////////////////////////////////////////////////////////////////////////////

#include "global.h"


//////////////////////////////////////////////////////////////////////////////
// helper functions
//

// load a string from registry into edit box
BOOL GetRegString(PWCHAR Reg,HWND Dlg, UINT Ctrl)
  { 
    WCHAR str[MAX_PATH];
    DWORD sl = MAX_PATH * sizeof(WCHAR);

    // load the string
    if(SHGetValue(REG_HIVE,REG_SETUP,Reg,NULL,str,&sl))
      return FALSE;

    // deliver
    SetDlgItemText(Dlg,Ctrl,str); 
    return TRUE; 
  }



// load a checkbox value
BOOL GetRegFlag(PWCHAR Reg, HWND Dlg, UINT Ctrl)
  { 
    DWORD fv = 0;  // flag value
    DWORD sl = sizeof(DWORD);
    
    // load value
    if (SHGetValue(REG_HIVE,REG_SETUP,Reg,NULL,&fv,&sl))
      return FALSE;
    
    // deliver
    CheckDlgButton(Dlg,Ctrl,fv);
    return TRUE; 
  }



// get byte count for registry saves
INT StrSize(PWCHAR str)
  { 
    return (lstrlen(str) + 1) * sizeof(WCHAR); 
  }



// save string from edit box to registry
BOOL SetRegString(PWCHAR Reg, HWND Dlg, UINT Ctrl)
  { 
    WCHAR str[MAX_PATH];

    GetDlgItemText(Dlg,Ctrl,str,MAX_PATH);
    if (SHSetValue(REG_HIVE,REG_SETUP,Reg,REG_SZ,str,StrSize(str)))
      return FALSE;
    return TRUE; 
  }



// save a checkbox value
BOOL SetRegFlag(PWCHAR Reg, HWND Dlg, UINT Ctrl) 
  { 
    DWORD fv = 0;   // flag value 

    fv = IsDlgButtonChecked(Dlg,Ctrl);
    if (SHSetValue(REG_HIVE,REG_SETUP,Reg,REG_DWORD,&fv,sizeof(DWORD)))
      return FALSE;
    else 
      return TRUE; 
  }



//////////////////////////////////////////////////////////////////////////////
// Load and Save settings
//

// load from registry to dialogs
BOOL LoadSettings(HWND Dlg)
  { 
    GetRegString(L"RootFolder",Dlg,4001);
    GetRegFlag(L"ConfineProjects",Dlg,4002);
    GetRegFlag(L"WorkspaceManager",Dlg,4003);
    GetRegFlag(L"NewProject",Dlg,4004);
    GetRegFlag(L"AddProjects",Dlg,4005);
    GetRegFlag(L"ProjectOptions",Dlg,4006);
    GetRegFlag(L"Dependencies",Dlg,4007);
    GetRegFlag(L"WorkFolder",Dlg,4011); 
    GetRegFlag(L"ResultFolder",Dlg,4012);
    GetRegFlag(L"ResultShell",Dlg,4013);
    return 1; 
  }



// save dialogs to registry
BOOL SaveSettings(HWND Dlg)
  { 
    SetRegString(L"RootFolder",Dlg,4001);
    SetRegFlag(L"ConfineProjects",Dlg,4002);
    SetRegFlag(L"WorkspaceManager",Dlg,4003);
    SetRegFlag(L"NewProject",Dlg,4004);
    SetRegFlag(L"AddProjects",Dlg,4005);
    SetRegFlag(L"ProjectOptions",Dlg,4006);
    SetRegFlag(L"Dependencies",Dlg,4007);
    SetRegFlag(L"WorkFolder",Dlg,4011); 
    SetRegFlag(L"ResultFolder",Dlg,4012);
    SetRegFlag(L"ResultShell",Dlg,4013);
    return EndDialog(Dlg,1); 
  }
 
  
  
//////////////////////////////////////////////////////////////////////////////
// Main Dialog functions
//

// choose a folder (no root)
BOOL ChooseFolder(HWND Dlg, UINT Ctrl )
  { 
    WCHAR pth[MAX_PATH] = {0};

    GetFolderName(Dlg,NULL,pth);
    SetDlgItemText(Dlg,Ctrl,pth);
    return 1; 
  }



// initialize tabs and sub-dialogs
BOOL InitSetup(HWND Dlg)
  { 
    LoadSettings(Dlg);  
    return TRUE; 
  }



// tosser for the main setup dialog
BOOL CALLBACK SetupTosser(HWND Dlg,UINT Msg,WPARAM wParm,LPARAM lParm)
  { 
    switch(Msg)
      { 
        case WM_INITDIALOG :
          return InitSetup(Dlg);
        case WM_COMMAND :
          switch(LOWORD(wParm))   
            { 
              case 4000 :
                return ChooseFolder(Dlg,4001);
              case IDOK :          // save button
                return SaveSettings(Dlg);
              case IDCANCEL :      // cancel button
                EndDialog(Dlg,0);
                return FALSE;
              default :
                return FALSE; 
            } 
        default :
          return FALSE; 
      } 
  }  



BOOL ShowSetup(HWND Parent)
  { 
    DialogBox(hMod,L"SETTINGS",Parent,(DLGPROC) SetupTosser); 
    return 1; 
  }
