/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                 Workspace Manager AddIn for Pelles POIDE                //
//                     Create and edit workspace files                     //
//                                                                         //
//                            By LD Blake, 2012                            //
/////////////////////////////////////////////////////////////////////////////

// workspace editor

#include "Global.h"
#include "Setup.h"
#include "Project.h"


// filters for open and add buttons
#define OPENFLT L"Workspace Files\0*.ppw\0\0"
#define ADDFLT  L"Project Files\0*.ppw;*.ppj\0\0"

//////////////////////////////////////////////////////////////////////////////
// Save and load PPW files
//

BOOL LoadWorkspaceFile(HWND Dlg,PTCHAR Name)
  { 
    WCHAR  cwd[MAX_PATH];     // current directory
    PWCHAR wfd;               // translated file data

    // setup to decode relative paths
    { 
      WCHAR fwd[MAX_PATH];    // path from filename

      GetCurrentDirectory(MAX_PATH,cwd);  
      lstrcpy(fwd,Name);
      PathRemoveFileSpec(fwd);
      SetCurrentDirectory(fwd); 
    }

    // load raw file from disk
    { 
      HANDLE  file;             // file handle
      PBYTE   raw;              // unprocessed file data
      DWORD   size;             // file size

      // open the file
      file = CreateFile(Name,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);          
      if (file == INVALID_HANDLE_VALUE)
        { 
          SetCurrentDirectory(cwd); 
          return FALSE; 
        }
      size  = GetFileSize(file,NULL);
      raw   = calloc(size + 5,sizeof(BYTE));
      wfd   = calloc(size + 5,sizeof(WCHAR));
      ReadFile(file,raw,size,&size,NULL);
      CloseHandle(file); 

      // convert to wide characters
      if (*(PWORD)raw == 0xFEFF)  // utf16le
        memcpy((PBYTE) wfd,raw,size);
      else                       // assume utf8 or ansi
        MultiByteToWideChar(CP_UTF8,0,(PCHAR) raw,size,wfd,size);
      free(raw); 
    }

    // tokenize and find ppj files
    { 
      PWCHAR  tok;              // token start
      PWCHAR  tint;             // used by wcstok
      WCHAR   path[MAX_PATH];   // converted paths

      tok = wcstok(wfd,L"\r\n",&tint);
      do
        { 
          if (! wcsncmp(tok,L"PROJECT = ",10))
            { 
              PathUnquoteSpaces(tok + 10);
              GetFullPathName(tok + 10,MAX_PATH,path,NULL);

              // check for dupes and add
              if (SendDlgItemMessage(Dlg,4005,LB_FINDSTRINGEXACT,-1,(LPARAM) path) == LB_ERR)
                SendDlgItemMessage(Dlg,4005,LB_ADDSTRING,0,(LPARAM) path); 
            } 
          tok = wcstok(NULL,L"\r\n",&tint); 
        }
      while(tok); 
    }

    // clean up
    free(wfd);
    SetCurrentDirectory(cwd);
    return TRUE; 
  }



// save the workspace to disk
BOOL SaveWorkspaceFile(HWND Dlg)
  { 
    WCHAR   name[MAX_NAME];       // workspace name
    WCHAR   path[MAX_PATH];       // workspace path
    INT     count;                // number of projects
    PWCHAR  fbuf;                 // file buffer

    // get string values
    GetDlgItemText(Dlg,4002,name,MAX_NAME);
    GetDlgItemText(Dlg,4004,path,MAX_PATH);
    // how many projects
    count = SendDlgItemMessage(Dlg,4005,LB_GETCOUNT,0,0);

    // need a name
    if (lstrlen(name) < 1)
      { 
        MessageBox(Dlg,L"Please enter a workspace name",L"OOPS!",MB_OK | MB_ICONWARNING);
        return 0; 
      }
    // need a path
    if (lstrlen(path) < 3)
      { 
        MessageBox(Dlg,L"Please enter a workspace path",L"OOPS!",MB_OK | MB_ICONWARNING);
        return 0; 
      }
    // need at least one project
    if ( count < 1)
      { 
        MessageBox(Dlg,L"Can't save empty workspaces",L"OOPS!",MB_OK | MB_ICONWARNING);
        return 0; 
      }

    // create buffer
    fbuf = calloc(MAX_PPW,sizeof(WCHAR));
    if (! fbuf)
      { 
        MessageBox(Dlg,L"Error creating workspace file",L"OOPS!",MB_OK | MB_ICONERROR);
        return 0; 
      }
 
    // add file header 
    lstrcpy(fbuf,L"# File created by Workspace Manager 2 for POIDE\r\n\r\n");
    lstrcat(fbuf,L"POC_WORKSPACE_VERSION = 1.00\r\n\r\n");

    // convert and write projects
    {
      WCHAR prj[MAX_PATH];

      // loop through projects list
      do 
        { 
          if (SendDlgItemMessage(Dlg,4005,LB_GETTEXT,--count,(LPARAM) prj) != LB_ERR)
            {    
              // convert to relative path
              MakeRelativePath(prj,path);
              swprintf(&fbuf[lstrlen(fbuf)],MAX_PATH,L"PROJECT = %ls\r\n",prj);
            } 

        }
      while (count > 0);
    }

    // finish
    lstrcat(fbuf,L"\r\n");   

    // add filename to path
    PathAddBackslash(path);
    lstrcat(path,name);
    lstrcat(path,L".ppw");
    DiskWrite(fbuf,path);              
    free(fbuf);  
    return 1; 
  }



//////////////////////////////////////////////////////////////////////////////
// Dialog Functions
//

// Open button
BOOL OpenWorkspace(HWND Dlg)
  { 
    TCHAR   name[MAX_PATH] = {0}; // filepath
    PTCHAR  fname;                // filename
    PTCHAR  ext;                  // extension
    
    RegLoadString(L"RootFolder", name);
    PathAddBackslash(name);
    wcscat(name, L"*.ppw");

    // launch dialog  
    if (! GetFileName(Dlg,OPENFLT,name))
      return 1;

    // check extension
    ext = PathFindExtension(name);
    if (_wcsicmp(ext,L".ppw"))
      { 
        MessageBox(Dlg,L"Only open .ppw files!", L"File",MB_OK | MB_ICONWARNING); 
        return 1; 
      } 

    // start new workspace
    SendDlgItemMessage(Dlg,4005,LB_RESETCONTENT,0,0);
    LoadWorkspaceFile(Dlg,name);
    fname = PathFindFileName(name);
    PathRemoveExtension(fname);
    SetDlgItemText(Dlg,4002,fname);
    PathRemoveFileSpec(name);
    SetDlgItemText(Dlg,4004,name); 
    return 1; 
  }     



// redirect file (create new workspace)
BOOL ChangeFolder(HWND Dlg)
  { 
    WCHAR name[MAX_PATH] = {0};
    WCHAR root[MAX_PATH] = {0};

    RegLoadString(L"RootFolder",root);

    if (GetFolderName(Dlg,root,name))
      SetDlgItemText(Dlg,4004,name);
    return 1; 
  }



// create a new project
BOOL MgrNewProject(HWND Dlg)
  { 
    NEWPROJECT prj;

    // prep the struct
    GetDlgItemText(Dlg,4004,prj.RootFolder,MAX_PATH);
    prj.Parent    = Dlg;  

    // and go
    if (! NewProject(&prj))
      return 0;

    // add to workspace list
    if (SendDlgItemMessage(Dlg,4005,LB_FINDSTRINGEXACT,-1,(LPARAM)prj.Path) == LB_ERR)
          SendDlgItemMessage(Dlg,4005,LB_ADDSTRING,0,(LPARAM) prj.Path); 
    return 1; 
  }



// Add button
BOOL AddToWorkspace(HWND Dlg)
  { 
    TCHAR   name[MAX_PATH] = {0}; // filepath
    PTCHAR  ext;                  // file extension

    // launch dialog and add
    if (! GetFileName(Dlg,ADDFLT,name))
      return 1; 
    // add to workspace
    ext = PathFindExtension(name);
    if (! _wcsicmp(ext,L".ppj"))  // check for dupes and add
      { 
        if (SendDlgItemMessage(Dlg,4005,LB_FINDSTRINGEXACT,-1,(LPARAM) name) == LB_ERR)
          SendDlgItemMessage(Dlg,4005,LB_ADDSTRING,0,(LPARAM) name); 
      }
    else if (! _wcsicmp(ext,L".ppw"))
      { 
        LoadWorkspaceFile(Dlg,name); 
      }
    else
      MessageBox(Dlg,L"Only add .ppj and .ppw files!", L"File",MB_OK | MB_ICONWARNING); 
    return 1; 
  } 


      
// Remove button
BOOL RemoveProjects(HWND Dlg)
  { 
    PINT  sel;      // selected items
    INT   csel;     // how many selected

    // get count
    csel = SendDlgItemMessage(Dlg,4005,LB_GETSELCOUNT,0,0);
    if (! csel)
      return 1;  

    // get list
    sel = calloc(csel,sizeof(INT));
    SendDlgItemMessage(Dlg,4005,LB_GETSELITEMS,csel,(LPARAM) sel);

    // trash away -- backwards to avoid index changes
    while(csel)
      SendDlgItemMessage(Dlg,4005,LB_DELETESTRING,sel[--csel],0); 

    free(sel); 
    return 1; 
  }

            

// Launch button
BOOL LaunchExit(HWND Dlg)
  { 
    TCHAR path[MAX_PATH + 10];

    if (! SaveWorkspaceFile(Dlg))
      return 1;

    GetDlgItemText(Dlg,4004,path,MAX_PATH);
    PathAddBackslash(path);
    GetDlgItemText(Dlg,4002,&path[lstrlen(path)],MAX_PATH - lstrlen(path));
    lstrcat(path,L".ppw");  
    AddIn_OpenDocument(hMain,AID_PROJECT,path);         
    return EndDialog(Dlg,1); 
  }



// Save button
BOOL SaveExit(HWND Dlg)
  { 
    if (! SaveWorkspaceFile(Dlg))
      return 1;
    return EndDialog(Dlg,1); 
  }



// drag and drop
BOOL HandleDroppedFiles(HWND Dlg, HANDLE hDrop)
  { 
    TCHAR name[MAX_PATH];   // dropped filename
    UINT  count = 0;        // drop count
    INT   x;                // counter

    // how many
    count = DragQueryFile(hDrop,0xFFFFFFFF,name,MAX_PATH);

    // process files in
    for (x = 0; x < count; x++)
      { 
        PTCHAR ext;
        DragQueryFile(hDrop,x,name,MAX_PATH);
        ext = PathFindExtension(name);
        
        // only accept ppj
        if (! _wcsicmp(ext,L".ppj"))
          { 
            if (SendDlgItemMessage(Dlg,4005,LB_FINDSTRINGEXACT,-1,(LPARAM) name) == LB_ERR)
              SendDlgItemMessage(Dlg,4005,LB_ADDSTRING,0,(LPARAM) name); 
          }
        if (! _wcsicmp(ext,L".ppw"))
          { 
            LoadWorkspaceFile(Dlg,name); 
          } 
      } 
    // release memory                        
    DragFinish(hDrop); 
    return 1; 
  }          



// initialize the path bar
BOOL InitWorkspace(HWND Dlg)
  { 
    WCHAR path[MAX_PATH] = {0};

    if(RegLoadString(L"RootFolder",path))
      SetDlgItemText(Dlg,4004,path);
    return 1; 
  }



// dialog tosser
BOOL CALLBACK WsTosser(HWND Dlg, UINT Msg, WPARAM wParm, LPARAM lParm)
  { 
    switch (Msg)
      { 
        case WM_INITDIALOG :        // initialization
          return InitWorkspace(Dlg);
        case WM_DROPFILES :         // drag and drop
          return HandleDroppedFiles(Dlg,(HANDLE) wParm);
        case WM_COMMAND :           // buttons
          switch (LOWORD(wParm))
            { 
              case 4001 :           // Open
                return OpenWorkspace(Dlg);
              case 4003 :           // Pick
                return ChangeFolder(Dlg);
              case 4006 :           // New
                return MgrNewProject(Dlg);
              case 4007 :           // Add
                return AddToWorkspace(Dlg);
              case 4008 :           // Remove
                return RemoveProjects(Dlg);
              case 4010 :           // Launch
                return LaunchExit(Dlg);
              case IDOK :           // Save
                return SaveExit(Dlg);
              case IDCANCEL :       // Cancel
                return EndDialog(Dlg, 0);  
              default :
                return 0; 
            }
        default :
          return 0; 
      } 
  } 


// launch the dialog
void EditWorkspace(HWND Parent)
  { 
    DialogBox(hMod,L"WSEDIT",Parent,(DLGPROC) WsTosser); 
  }


