﻿///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                    Web features for Pelles C ver 7.00+                    //
//                             By LD Blake, 2012                             //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

// web favorites

#include "webFaves.h"
#include <mmsystem.h>


///////////////////////////////////////////////////////////////////////////////
// Add favorites dialog
//

// initialize the Add dialog
BOOL InitNewFave (HWND Dlg, HWND mdi)
  { 
    INT     ts;       // size of page text
    PWCHAR  buf;      // text buffer for page
    PWCHAR  ptl;      // points to title
    INT     idx = 0;  // title array index 
    WCHAR   title[MAX_PATH + 1] = {0};  // title text
    ADDIN_WEB_INFO wi = {0};            // url info

    // get page sized buffer
    ts = AddIn_GetHtmlTextLength(mdi) + 4;
    if (ts > 0)
      {
        buf = calloc(ts,sizeof(WCHAR));
        AddIn_GetHtmlText(mdi,buf,ts);

        // find and copy title
        ptl = _wcsistr(buf,L"<TITLE>");
        if (ptl)
          {   
            ptl = wcschr(ptl,L'>') + 1;
            while (*ptl && (*ptl != L'<') &&(idx < MAX_PATH))
              title[idx++] = *ptl++; 
            title[idx] = 0;
          }
        // stuff it in the edit
        free(buf);
        SetDlgItemText(Dlg,4002,title);   
      }
    // get url
    wi.cbSize = sizeof(wi);
    AddIn_GetWebInfo(mdi,&wi);
    if (wcslen(wi.pszURL) > 5)
      SetDlgItemText(Dlg,4001,wi.pszURL);

    return 0;
  }



// save favorite
BOOL SaveFavorite (HWND Dlg)
  {
    WCHAR url[MAX_PATH] = {0};  // url
    WCHAR ttl[MAX_PATH] = {0};  // page title
    // get from dialog
    GetDlgItemText(Dlg,4001,url,MAX_PATH);
    GetDlgItemText(Dlg,4002,ttl,MAX_PATH);

    // put value in registry
    SHSetValue(REG_HIVE,REG_FAVE,ttl,REG_SZ,url,(wcslen(url) + 1) * sizeof(WCHAR));

    // and exit
    return EndDialog(Dlg,1);
  }



// settings dialog message tosser
BOOL CALLBACK AddTosser (HWND Dlg, UINT Msg, WPARAM wParm, LPARAM lParm)
  { 
    switch (Msg)
      { 
        case WM_INITDIALOG :
          return InitNewFave(Dlg,(HWND) lParm);
        case WM_COMMAND :
          switch (LOWORD(wParm))
            { 
              case IDOK :                   // OK
                return SaveFavorite(Dlg);
              case IDCANCEL :               // Cancel
                return EndDialog(Dlg,0);
              default :
                return FALSE; 
            }
        default :
          return FALSE; 
      } 
  }



// save in registry
BOOL AddFavorite (HWND Parent)
  {
    HWND mdi;   // current mdi window

    // get the active mdi window
    mdi = AddIn_GetActiveDocument(hMain);
    // get the window's file type
    if (AddIn_GetDocumentType(mdi) != AID_WEB)
      {
        PlaySound(L"SystemExclamation",NULL,SND_ALIAS | SND_ASYNC);
        return 0;
      }

    return DialogBoxParam(hInst,L"ADDFAVE",Parent,(DLGPROC) AddTosser,(LPARAM) mdi);  
  }  



////////////////////////////////////////////////////////////////////////////////
// Favorites picker dialog
//


// go to selected URL
BOOL LaunchURL (HWND Dlg, BOOL New)
  { 
    INT idx;              // selection index
    WCHAR ttl[MAX_PATH];  // title from listbox
    WCHAR url[MAX_PATH];  // url for web
    DWORD tp;             // registry type
    DWORD sz;             // buffer size
    HWND  mdi;            // web window handle
    ADDIN_WEB_INFO wi = {0}; // web info struct

    // get title from listbox
    idx = SendDlgItemMessage(Dlg,4001,LB_GETCURSEL,0,0);
    if (idx == LB_ERR)
      {
        PlaySound(L"SystemExclamation",NULL,SND_ALIAS | SND_ASYNC);
        return 1;
      }   
    SendDlgItemMessage(Dlg,4001, LB_GETTEXT,idx,(LPARAM) ttl);

    // get url from registry
    sz = MAX_PATH * sizeof(WCHAR);
    SHGetValue(REG_HIVE,REG_FAVE,ttl,&tp,url,&sz);

    // get current window information
    mdi = AddIn_GetActiveDocument(hMain);
    // make sure we have a web window
    if ((AddIn_GetDocumentType(mdi) != AID_WEB) || New)
       AddIn_OpenDocument(hMain,AID_WEB,url);
    else
      { 
        wi.cbSize = sizeof(wi);
        wi.pszURL = url;
        wi.eEncryption  = ADDIN_WEB_ENCRYPT_UNKNOWN;
        wi.fWelcome = 0;
        AddIn_SetWebInfo(mdi,&wi);
      }

    EndDialog(Dlg,1);
    return 1;
  }



// double click handler for listbox
BOOL ListDoubleClick (HWND Dlg, WORD Notice)
  {
    if (Notice != LBN_DBLCLK)
      return 1;

    return LaunchURL (Dlg,0);
  }



// initialize the favorites list
BOOL InitListBox (HWND Dlg)
  {
    WCHAR ttl[MAX_PATH];  // titles buffer
    DWORD tsz;            // title size  
    INT   idx = 0;        // enum index
    HKEY  hk;             // search key

    // open registry key
    RegOpenKey(REG_HIVE,REG_FAVE,&hk);
    
    // get names list
    tsz = MAX_PATH;
    while (! RegEnumValue(hk,idx,ttl,&tsz,NULL,NULL,NULL,NULL))
      { 
        if (wcslen(ttl) > 0)
          SendDlgItemMessage(Dlg,4001,LB_ADDSTRING,0,(LPARAM) ttl);
        idx++;
        tsz = MAX_PATH;
      }

    RegCloseKey(hk);
    return 0;
  }  



// add button
BOOL ListAddFavorite (HWND Dlg)
  {
    if (AddFavorite(Dlg))
      { 
        SendDlgItemMessage(Dlg,4001,LB_RESETCONTENT,0,0);
        InitListBox(Dlg);
      }
    return 1;
  }


// destroy list item
BOOL ListDelFavorite (HWND Dlg)
  {
    INT idx;              // selected item
    WCHAR ttl[MAX_PATH];  // selected text

    // get selected item
    idx = SendDlgItemMessage(Dlg,4001,LB_GETCURSEL,0,0);
    if (idx == LB_ERR)
      return 1;

    // get item text
    SendDlgItemMessage(Dlg,4001,LB_GETTEXT,idx,(LPARAM) ttl);
    SendDlgItemMessage(Dlg,4001,LB_DELETESTRING,idx,0);
  
    // remove from registry
    SHDeleteValue(REG_HIVE,REG_FAVE,ttl);

    return 1;
  }



// settings dialog message tosser
BOOL CALLBACK FaveTosser(HWND Dlg, UINT Msg, WPARAM wParm, LPARAM lParm)
  { 
    switch (Msg)
      { 
        case WM_INITDIALOG :
          return InitListBox(Dlg);
        case WM_COMMAND :
          switch (LOWORD(wParm))
            { 
              case 4001 :                   // mouse activate
                return ListDoubleClick(Dlg,HIWORD(wParm));
              case 4002 :                   // add
                return ListAddFavorite(Dlg);
              case 4003 :
                return ListDelFavorite(Dlg);
              case IDOK :                   // OK
                return LaunchURL(Dlg,0);
              case IDCANCEL :               // Cancel
                return EndDialog(Dlg,0);
              default :
                return FALSE; 
            }
        default :
          return FALSE; 
      } 
  }



// launch the favorites dialog
BOOL ShowFavorites(HWND Parent)
  {
    DialogBox(hInst,L"FAVORITES",Parent,(DLGPROC) FaveTosser);
    return 1;
  }


