﻿///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                    Web features for Pelles C ver 7.00+                    //
//                             By LD Blake, 2012                             //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

// main and settings

#include "webFaves.h"


// longest word for searches
#define MAX_WORD 32

// button values  
#define BTN_START 100
#define BTN_HOME  101
#define BTN_MSDN  102
#define BTN_WEB   103
#define BTN_FAVE  104
// source context menu 
#define MNU_MSDN  110
#define MNU_WEB   111
// web context menu
#define MNU_FAVE  120
#define MNU_FNEW  130


// globals
HINSTANCE hInst;  // dll instance handle
HWND      hMain;  // ide window handle


// settings structure
struct t_SETTING
  { 
    DWORD F1Enable  : 1,
          F1Web     : 1,
          F1Extern  : 1,
          StartPage : 1,
          HomePage  : 1,
          SrchMsdn  : 1,
          SrchWeb   : 1,
          WebFaves  : 1;
    WCHAR MsdnUrl[MAX_PATH];
    WCHAR WebUrl[MAX_PATH]; 
  } Setting;



////////////////////////////////////////////////////////////////////////////////
// Add/Remove features
//

BOOL AddWebFeatures(void)
  { 
    ADDIN_ADD_COMMAND ac;  // add command structure

    // preconditions
    ac.cbSize = sizeof(ac);
    ac.idMenu = AIM_MENU_NOTHING; // toolbar

    // add buttons
    if (Setting.StartPage)
      { 
        ac.hIcon    = LoadImage(hInst,L"START",IMAGE_ICON,16,16,LR_LOADTRANSPARENT);
        ac.id       = BTN_START;
        ac.pszText  = L"Open internal start page";
        AddIn_AddCommand(hMain,&ac); 
      }
    if (Setting.HomePage)
      { 
        ac.hIcon    = LoadImage(hInst,L"HOME",IMAGE_ICON,16,16,LR_LOADTRANSPARENT);
        ac.id       = BTN_HOME;
        ac.pszText  = L"Open internal home page";
        AddIn_AddCommand(hMain,&ac); 
      }
    if (Setting.SrchMsdn)
      { 
        ac.hIcon    = LoadImage(hInst,L"MSDN",IMAGE_ICON,16,16,LR_LOADTRANSPARENT);
        ac.id       = BTN_MSDN;
        ac.pszText  = L"Search on MSDN";
        AddIn_AddCommand(hMain,&ac); 
      }
    if (Setting.SrchWeb)
      { 
        ac.hIcon    = LoadImage(hInst,L"WEB",IMAGE_ICON,16,16,LR_LOADTRANSPARENT);
        ac.id       = BTN_WEB;
        ac.pszText  = L"Search on the Web";
        AddIn_AddCommand(hMain,&ac); 
      } 
    if (Setting.WebFaves)
      {
        ac.hIcon    = LoadImage(hInst,L"FAVES",IMAGE_ICON,16,16,LR_LOADTRANSPARENT);
        ac.id       = BTN_FAVE;
        ac.pszText  = L"Open web favorites";
        AddIn_AddCommand(hMain,&ac);
      }

    // source menu 
    ac.idMenu = AIM_MENU_SOURCE; // source editor
    ac.hIcon    = LoadImage(hInst,L"MSDN",IMAGE_ICON,16,16,LR_LOADTRANSPARENT);
    ac.id       = MNU_MSDN;
    ac.pszText  = L"Find on MSDN";
    AddIn_AddCommand(hMain,&ac); 
    ac.hIcon    = LoadImage(hInst,L"WEB",IMAGE_ICON,16,16,LR_LOADTRANSPARENT);
    ac.id       = MNU_WEB;
    ac.pszText  = L"Find on the Web";
    AddIn_AddCommand(hMain,&ac); 

    // web menu
    ac.idMenu = AIM_MENU_WEB;
    ac.hIcon    = LoadImage(hInst,L"FAVES",IMAGE_ICON,16,16,LR_LOADTRANSPARENT);
    ac.id       = MNU_FAVE;
    ac.pszText  = L"Open web favorites";
    AddIn_AddCommand(hMain,&ac);
    ac.id       = MNU_FNEW;
    ac.pszText  = L"Add to web favorites";
    AddIn_AddCommand(hMain,&ac);
    return TRUE;
  } 



BOOL RemoveWebFeatures(void)
  { 
    AddIn_RemoveCommand(hMain,BTN_START);
    AddIn_RemoveCommand(hMain,BTN_HOME);
    AddIn_RemoveCommand(hMain,BTN_MSDN);
    AddIn_RemoveCommand(hMain,BTN_WEB);
    AddIn_RemoveCommand(hMain,BTN_FAVE);
    AddIn_RemoveCommand(hMain,MNU_MSDN);
    AddIn_RemoveCommand(hMain,MNU_WEB);
    AddIn_RemoveCommand(hMain,MNU_FAVE);
    AddIn_RemoveCommand(hMain,MNU_FNEW);

    return TRUE; 
  }



///////////////////////////////////////////////////////////////////////////////
// Handle Settings
//

BOOL SaveSettings(HWND Dlg)
  { 
    Setting.F1Enable  = IsDlgButtonChecked(Dlg,4000);
    Setting.F1Web     = IsDlgButtonChecked(Dlg,4002); 
    Setting.F1Extern  = IsDlgButtonChecked(Dlg,4003);
    Setting.StartPage = IsDlgButtonChecked(Dlg,4004);
    Setting.HomePage  = IsDlgButtonChecked(Dlg,4005);
    Setting.SrchMsdn  = IsDlgButtonChecked(Dlg,4006);
    Setting.SrchWeb   = IsDlgButtonChecked(Dlg,4007);
    Setting.WebFaves  = IsDlgButtonChecked(Dlg,4008);
    GetDlgItemText(Dlg,4009,Setting.MsdnUrl,MAX_PATH);
    GetDlgItemText(Dlg,4010,Setting.WebUrl,MAX_PATH);

    if (!SHSetValue(REG_HIVE,REG_SET,L"Settings",REG_BINARY,&Setting,sizeof(Setting))) 
      { 
        EndDialog(Dlg,1); 
        return TRUE; 
      }
    MessageBox(Dlg,L"Error saving Settings",L"OOPS!",MB_ICONWARNING | MB_OK);
    return TRUE; }



BOOL LoadSettings(void)
  { 
    DWORD dt;                   // dummy for data type
    DWORD ds = sizeof(Setting); // struct size

    if (SHGetValue(REG_HIVE,REG_SET,L"Settings",&dt,&Setting,&ds))
      return FALSE;
    return TRUE; 
  }



VOID EnableF1Choices(HWND Dlg,BOOL Enable)
  {  
    EnableWindow(GetDlgItem(Dlg,4001),Enable);
    EnableWindow(GetDlgItem(Dlg,4002),Enable); 
  }



// prepare settings editor   
BOOL ImportSettings(HWND Dlg)
  { 
    RemoveWebFeatures();
    CheckDlgButton(Dlg,4000,Setting.F1Enable);

    if (!Setting.F1Web)
      CheckDlgButton(Dlg,4001,1);
    else
      CheckDlgButton(Dlg,4002,1);

    // enable sub choice buttons
    EnableF1Choices(Dlg,Setting.F1Enable);
    CheckDlgButton(Dlg,4003,Setting.F1Extern);
    CheckDlgButton(Dlg,4004,Setting.StartPage);
    CheckDlgButton(Dlg,4005,Setting.HomePage);
    CheckDlgButton(Dlg,4006,Setting.SrchMsdn);
    CheckDlgButton(Dlg,4007,Setting.SrchWeb);
    CheckDlgButton(Dlg,4008,Setting.WebFaves);

    if(lstrlen(Setting.MsdnUrl) > 0)
      SetDlgItemText(Dlg,4009,Setting.MsdnUrl);

    if(lstrlen(Setting.WebUrl) > 0)
      SetDlgItemText(Dlg,4010,Setting.WebUrl);
    
    SetFocus(GetDlgItem(Dlg,4000));
    return FALSE; 
  }



// settings dialog message tosser
BOOL CALLBACK SetupTosser(HWND Dlg, UINT Msg, WPARAM wParm, LPARAM lParm)
  { 
    switch (Msg)
      { 
        case WM_INITDIALOG :
          return ImportSettings(Dlg);
        case WM_COMMAND :
          switch (LOWORD(wParm))
            { 
              case IDOK :                   // OK
                return SaveSettings(Dlg);
              case IDCANCEL :               // Cancel
                return EndDialog(Dlg,0);
              case 4000 :                   // enable sub choices
                EnableF1Choices(Dlg,IsDlgButtonChecked(Dlg,4000));
                return TRUE;
              default :
                return FALSE; 
            }
        default :
          return FALSE; 
      } 
  }



// launch the Settings dialog
INT EditSettings(HWND hWin)
  { 
    INT ret;  // dialog return code
    ret = DialogBox(hInst,L"SETUP",hWin,(DLGPROC) SetupTosser); 

    AddWebFeatures();
    return ret; 
  }



///////////////////////////////////////////////////////////////////////////////
// Setup dialog launcher
//

ADDINAPI VOID WINAPI AddInSetup(HWND hWin,LPCVOID Nothing)
  { 
    EditSettings(hWin); 
  }



///////////////////////////////////////////////////////////////////////////////
// F1 searches
//

ADDINAPI BOOL WINAPI AddInHelp(HWND hWin,ADDIN_HELPEVENT Event,LPCVOID Data)
  { 
    WCHAR url[MAX_PATH] = {0};  // url string

    // let local search complete first
    if (Event == AIHE_SRC_KEYWORD_FIRST)    
      return FALSE;

    // bail if not checked
    if (!Setting.F1Enable)    
      return FALSE;

    // get basic url
    if (Setting.F1Web)
      wcsncpy(url,Setting.WebUrl,MAX_PATH);
    else
      wcsncpy(url,Setting.MsdnUrl,MAX_PATH);

    // add keyword
    wcscat(url,(PWCHAR)Data);

    // and search
    if (Setting.F1Extern)
      { ShellExecute(hWin,L"Open",url,NULL,NULL,SW_SHOW);
        return TRUE; }

    // use internal browser
    AddIn_OpenDocument(hMain,AID_WEB,url); 
    return TRUE; 
  }  



///////////////////////////////////////////////////////////////////////////////
// Button handlers
//

VOID ShowInternalHome(void)
  { 
    WCHAR url[MAX_PATH] = {0}; // url string
    DWORD dt;                 // dummy type
    DWORD ds = MAX_PATH;      // buffer size

    // load from registry
    SHGetValue(REG_HIVE,REG_HOME,L"HomePage",&dt,url,&ds);

    // display
    AddIn_OpenDocument(hMain,AID_WEB,url); 
  }



VOID SearchOnline(BOOL Web)
  { 
    WCHAR url[MAX_PATH] = {0};  // url string
    WCHAR word[MAX_WORD] = {0}; // current editor word
    HWND  cw;                   // active document

    // get url from settings
    if(Web)
      wcsncpy(url,Setting.WebUrl,MAX_PATH);
    else  
      wcsncpy(url,Setting.MsdnUrl,MAX_PATH);

    // get current word in editor
    cw = AddIn_GetActiveDocument(hMain);
    AddIn_GetSourceWord(cw,word,MAX_WORD);

    // add to url
    wcscat(url,word);

    // display
    AddIn_OpenDocument(hMain,AID_WEB,url); 
  }



// button command tosser
ADDINAPI void WINAPI AddInCommandEx(INT Cmd, LPCVOID Data)
  { 
    switch (Cmd)
      { 
        case BTN_START :
          AddIn_SendIDECommand(hMain,AIC_VIEW_SHOWWELCOME);
          return;
        case BTN_HOME :
          ShowInternalHome();
          return;
        case BTN_MSDN :
        case MNU_MSDN :
          SearchOnline(0);
          return;  
        case BTN_WEB :
        case MNU_WEB :
          SearchOnline(1);
          return;
        case BTN_FAVE :
        case MNU_FAVE :
          ShowFavorites(hMain);
          return;  
        case MNU_FNEW :
          AddFavorite(hMain);
          return;
        default :
          return; 
      } 
  }



///////////////////////////////////////////////////////////////////////////////
// Entry points
//

// main addin procedure
ADDINAPI BOOL WINAPI AddInMain(HWND hWin, ADDIN_EVENT Event)
  { 
    switch (Event)
      { 
        case AIE_APP_CREATE:
          hMain = hWin;
          if (! LoadSettings())
            EditSettings(hWin);
          else
            AddWebFeatures();
          return TRUE;
        case AIE_APP_DESTROY :
          return RemoveWebFeatures();
        default:
            return TRUE;  
      } 
  }



// dll load entry point
BOOL WINAPI DllMain(HANDLE hDLL, DWORD dwReason, LPVOID lpReserved)
  { 
    switch (dwReason)
      { 
        case DLL_PROCESS_ATTACH:
          hInst = hDLL;
          return TRUE;
        case DLL_PROCESS_DETACH:
          return TRUE;
        default:
          return TRUE; 
      } 
  }

